/***************************************************************************
 * nbase_winunix.h -- Misc. compatability routines that generally try to   *
 * reproduce UNIX-centric concepts on Windows.                             *
 *                                                                         *
 ***********************IMPORTANT NMAP LICENSE TERMS************************
 *                                                                         *
 * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
 * program is free software; you may redistribute and/or modify it under   *
 * the terms of the GNU General Public License as published by the Free    *
 * Software Foundation; Version 2.  This guarantees your right to use,     *
 * modify, and redistribute this software under certain conditions.  If    *
 * you wish to embed Nmap technology into proprietary software, we may be  *
 * willing to sell alternative licenses (contact sales@insecure.com).      *
 * Many security scanner vendors already license Nmap technology such as   *
 * our remote OS fingerprinting database and code.                         *
 *                                                                         *
 * Note that we consider aggregation/inclusion/integration of Nmap into an *
 * executable installer to constitute a derived work and thus subject to   *
 * the GPL restrictions.  We also consider certain programs that tightly   *
 * integrate with Nmap to constitute derivative works, even if they only   *
 * interface with Nmap by executing the Nmap binary and interpreting its   *
 * output rather than by direct linking.  If you are interested in         *
 * including Nmap with your proprietary software or appliance, please      *
 * contact us first to ensure proper licensing.                            *
 *                                                                         *
 * If you received these files with a written license agreement or         *
 * contract stating terms other than the (GPL) terms above, then that      *
 * alternative license agreement takes precedence over these comments.     *
 *                                                                         *
 * Source is provided to this software because we believe users have a     *
 * right to know exactly what a program is going to do before they run it. *
 * This also allows you to audit the software for security holes (none     *
 * have been found so far).                                                *
 *                                                                         *
 * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 * and add new features.  You are highly encouraged to send your changes   *
 * to fyodor@insecure.org for possible incorporation into the main         *
 * distribution.  By sending these changes to Fyodor or one the            *
 * Insecure.Org development mailing lists, it is assumed that you are      *
 * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
 * to reuse, modify, and relicense the code.  Nmap will always be          *
 * available Open Source, but this is important because the inability to   *
 * relicense code has caused devastating problems for other Free Software  *
 * projects (such as KDE and NASM).  We also occasionally relicense the    *
 * code to third parties as discussed above.  If you wish to specify       *
 * special license conditions of your contributions, just say so when you  *
 * send them.                                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License for more details at                              *
 * http://www.gnu.org/copyleft/gpl.html .                                  *
 *                                                                         *
 ***************************************************************************/

/* $Id: nbase_winunix.h,v 1.2 2003/09/06 09:10:55 fyodor Exp $ */

#ifndef NBASE_WINUNIX_H
#define NBASE_WINUNIX_H

#define _INC_ERRNO  /* supress errno.h */

//	Supress winsock.h
#define _WINSOCKAPI_

#include <windows.h>
#include <winsock2.h>
/* #include <ws2tcpip.h> // IPv6 stuff */
#include <time.h>
#include <iptypes.h>
#include <stdlib.h>
#include <malloc.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <process.h>
//#include <errno.h>
#include <limits.h>
#include <WINCRYPT.H>
#include <math.h>
//#include <packet_types.h>

typedef unsigned char u_int8_t;
typedef unsigned short u_int16_t;

typedef unsigned long u_int32_t;
typedef unsigned int ssize_t;

//#define HAVE_STRUCT_IP
//#define HAVE_STRUCT_ICMP

#define SIOCGIFCONF     0x8912          /* get iface list */

#ifndef GLOBALS
#define GLOBALS 1

//extern struct interface_info global_adapter;
#endif

/* Disables VC++ warning:
  "integral size mismatch in argument; conversion supplied".  Perhaps
  I should try to fix this with casts at some point */
// #pragma warning(disable: 4761)

/* #define signal(x,y) ((void)0)	// ignore for now
                                // later release may set console handlers
*/

extern struct winops wo;

#define munmap(ptr, len) win32_munmap(ptr, len)
int nmapwin_isroot();

/* Windows error message names */
#define ECONNABORTED    WSAECONNABORTED
#define ECONNRESET      WSAECONNRESET
#define ECONNREFUSED    WSAECONNREFUSED
#define EAGAIN		WSAEWOULDBLOCK
#define EHOSTUNREACH	WSAEHOSTUNREACH
#define ENETDOWN	WSAENETDOWN
#define ENETUNREACH	WSAENETUNREACH
#define ENETRESET	WSAENETRESET
#define ETIMEDOUT	WSAETIMEDOUT
#define EHOSTDOWN	WSAEHOSTDOWN
#define EINPROGRESS	WSAEINPROGRESS
#define EINVAL          WSAEINVAL      /* Invalid argument */
#define EPERM           WSAEACCES      /* Operation not permitted */
#define EINTR           WSAEINTR      /* Interrupted system call */
#define ENOBUFS         WSAENOBUFS     /* No buffer space available */
#define ENOENT          WSAENOENT      /* No such file or directory */
#define EMSGSIZE        WSAEMSGSIZE    /* Message too long */

#define close(x) my_close(x)
#define read(x,y,z) recv(x,(char*)(y),z,0)
int my_close(int sd);

typedef unsigned short u_short_t;

int win32_sendto(int sd, const char *packet, int len, 
	   unsigned int flags, struct sockaddr *to, int tolen);

int win32_socket(int af, int type, int proto);

#define socket(af, type, proto) win32_socket(af, type, proto)
#define sendto(sd, packet, len, flags, to, tolen) win32_sendto(sd, packet, len, flags, to, tolen)

#endif /* NBASE_WINUNIX_H */
